//===========================================================================
//  $Name: arts++-1-1-a13 $
//  $Id: ArtsInterfaceMatrixEntry.hh,v 1.2 2004/04/21 23:51:26 kkeys Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSINTERFACEMATRIXENTRY_HH_
#define _ARTSINTERFACEMATRIXENTRY_HH_

extern "C" {
#include "artslocal.h"
#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>
  
#include "caida_t.h"
}

#ifdef HAVE_IOSTREAM
  #include <iostream>
#else
  #include <iostream.h>
#endif
#ifdef HAVE_IOMANIP
  #include <iomanip>
#else
  #include <iomanip.h>
#endif

#include "ArtsPrimitive.hh"

//---------------------------------------------------------------------------
//  class ArtsInterfaceMatrixEntry
//---------------------------------------------------------------------------
//!  This class represents a single interface matrix entry.  It holds
//!  a source interface index, a destination interface index, and the
//!  packets and bytes sent from the source interface to the destination
//!  interface.
//---------------------------------------------------------------------------
class ArtsInterfaceMatrixEntry
{
public:
  //-------------------------------------------------------------------------
  //                       ArtsInterfaceMatrixEntry()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsInterfaceMatrixEntry();
  
  //-------------------------------------------------------------------------
  //                       ~ArtsInterfaceMatrixEntry()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsInterfaceMatrixEntry();
  
  //-------------------------------------------------------------------------
  //                       uint8_t Descriptor() const
  //.........................................................................
  //!  Returns the descriptor field of the object.
  //-------------------------------------------------------------------------
  uint8_t Descriptor() const;
  
  //-------------------------------------------------------------------------
  //                 uint8_t Descriptor(uint8_t descriptor)
  //.........................................................................
  //!  Sets the descriptor field of the object.  Normally not used
  //!  directly in applications.
  //-------------------------------------------------------------------------
  uint8_t Descriptor(uint8_t descriptor);
  
  //-------------------------------------------------------------------------
  //                          uint16_t Src() const
  //.........................................................................
  //!  Returns the source interface index of the object.
  //-------------------------------------------------------------------------
  uint16_t Src() const;
  
  //-------------------------------------------------------------------------
  //                       uint16_t Src(uint16_t src)
  //.........................................................................
  //!  Sets and returns the source interface index of the object.
  //-------------------------------------------------------------------------
  uint16_t Src(uint16_t src);
  
  //-------------------------------------------------------------------------
  //                          uint16_t Dst() const
  //.........................................................................
  //!  Returns the destination interface index of the object.
  //-------------------------------------------------------------------------
  uint16_t Dst() const;
  
  //-------------------------------------------------------------------------
  //                       uint16_t Dst(uint16_t dst)
  //.........................................................................
  //!  Sets and returns the destination interface index of the object.
  //-------------------------------------------------------------------------
  uint16_t Dst(uint16_t dst);
  
  //-------------------------------------------------------------------------
  //                          uint64_t Pkts() const
  //.........................................................................
  //!  Returns the packet count for the object.
  //-------------------------------------------------------------------------
  uint64_t Pkts() const;
  
  //-------------------------------------------------------------------------
  //                      uint64_t Pkts(uint64_t pkts)
  //.........................................................................
  //!  Sets and returns the packet count for the object.
  //-------------------------------------------------------------------------
  uint64_t Pkts(uint64_t pkts);
  
  //-------------------------------------------------------------------------
  //                         uint64_t Bytes() const
  //.........................................................................
  //!  Returns the byte count for the object.
  //-------------------------------------------------------------------------
  uint64_t Bytes() const;
  
  //-------------------------------------------------------------------------
  //                     uint64_t Bytes(uint64_t bytes)
  //.........................................................................
  //!  Sets and returns the byte count for the object.
  //-------------------------------------------------------------------------
  uint64_t Bytes(uint64_t bytes);
  
  //-------------------------------------------------------------------------
  //               uint32_t Length(uint8_t version = 0) const
  //.........................................................................
  //!  Returns the number of bytes needed to store the object in a file.
  //-------------------------------------------------------------------------
  uint32_t Length(uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //            std::istream & read(std::istream& is, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsInterfaceMatrixEntry from an istream.  Returns the
  //!  istream.
  //-------------------------------------------------------------------------
  std::istream & read(std::istream& is, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //                  int read(int fd, uint8_t version = 0)
  //.........................................................................
  //!  Reads an ArtsInterfaceMatrixEntry from a file descriptor.  Returns
  //!  the number of bytes read on success, -1 on failure.
  //-------------------------------------------------------------------------
  int read(int fd, uint8_t version = 0);
  
  //-------------------------------------------------------------------------
  //         std::ostream & write(std::ostream& os, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsInterfaceMatrixEntry to an ostream.  Returns the
  //!  ostream.
  //-------------------------------------------------------------------------
  std::ostream & write(std::ostream& os, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //              int write(int fd, uint8_t version = 0) const
  //.........................................................................
  //!  Writes an ArtsInterfaceMatrixEntry to a file descriptor.  Returns
  //!  the number of bytes written on success, -1 on failure.
  //-------------------------------------------------------------------------
  int write(int fd, uint8_t version = 0) const;
  
  //-------------------------------------------------------------------------
  //        friend std::ostream &
  //        operator << (std::ostream& os,               
  //                     const ArtsInterfaceMatrixEntry & ifEntry)
  //.........................................................................
  //!  Overloaded '<<' operator to dump the contents of an
  //!  ArtsInterfaceMatrixEntry to an ostream in a human-readable form.
  //!  Returns the ostream.
  //-------------------------------------------------------------------------
  friend std::ostream &
  operator << (std::ostream& os,
               const ArtsInterfaceMatrixEntry & ifEntry);

  #ifndef NDEBUG
    //------------------------------------------------------------------------
    //                       static uint32_t NumObjects()                      
    //........................................................................
    //  
    //------------------------------------------------------------------------
    static uint32_t NumObjects()
    {
      return(_numObjects);
    }
  #endif  // NDEBUG
  
private:
  uint8_t    _descriptor;
  uint16_t   _src;
  uint16_t   _dst;
  uint64_t   _pkts;
  uint64_t   _bytes;

  static uint32_t  _numObjects;
};

//---------------------------------------------------------------------------
//  class ArtsInterfaceMatrixEntryGreaterBytes
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting
//!  ArtsInterfaceMatrixEntry objects in descending order by bytes of
//!  traffic.
//---------------------------------------------------------------------------
class ArtsInterfaceMatrixEntryGreaterBytes
{
public:
  //---------------------------------------------------------------------------
  // bool operator () (const ArtsInterfaceMatrixEntry & interfaceEntry1,
  //                   const ArtsInterfaceMatrixEntry & interfaceEntry2) const
  //...........................................................................
  //
  //---------------------------------------------------------------------------
  bool operator () (const ArtsInterfaceMatrixEntry & interfaceEntry1,
                    const ArtsInterfaceMatrixEntry & interfaceEntry2) const;
};

//---------------------------------------------------------------------------
//  class ArtsInterfaceMatrixEntryGreaterPkts
//---------------------------------------------------------------------------
//!  This class is used as a comparison operator for sorting
//!  ArtsInterfaceMatrixEntry objects in descending order by packets of
//!  traffic.
//---------------------------------------------------------------------------
class ArtsInterfaceMatrixEntryGreaterPkts
{
public:
  //---------------------------------------------------------------------------
  // bool operator () (const ArtsInterfaceMatrixEntry & interfaceEntry1,
  //                   const ArtsInterfaceMatrixEntry & interfaceEntry2) const
  //...........................................................................
  //
  //---------------------------------------------------------------------------
  bool operator () (const ArtsInterfaceMatrixEntry & interfaceEntry1,
                    const ArtsInterfaceMatrixEntry & interfaceEntry2) const;
};

#endif  // _ARTSINTERFACEMATRIXENTRY_HH_
